// generated by Fast Light User Interface Designer (fluid) version 1.0304

#include "PADnoteUI.h"
// PADNoteUI.cc
// Original ZynAddSubFX author Nasca Octavian Paul
// Copyright (C) 2002-2005 Nasca Octavian Paul
// Copyright 2009-2011, Alan Calvert
// Copyright 2015-2017, Will Godfrey

// This file is part of yoshimi, which is free software: you can redistribute
// it and/or modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.

// yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.   See the GNU General Public License (version 2 or
// later) for more details.

// You should have received a copy of the GNU General Public License along with
// yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
// Street, Fifth Floor, Boston, MA  02110-1301, USA.

// This file is derivative of ZynAddSubFX original code

// Modified October 2017

#include <FL/Fl_Box.H>
    #include <FL/Fl_Group.H>
    #include <FL/Fl_File_Chooser.H>
    #include <FL/fl_draw.H>
    #include "Params/PADnoteParameters.h"
    #include "Misc/MiscFuncs.h"
    #include "UI/MiscGui.h"
    #include "PresetsUI.h"
    #include "UI/WidgetPDial.h"
    #include "EnvelopeUI.h"
    #include "LFOUI.h"
    #include "FilterUI.h"
#include "MasterUI.h"

PADnoteHarmonicProfile::PADnoteHarmonicProfile(int x,int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label) {
  pars=NULL;
}

void PADnoteHarmonicProfile::init(PADnoteParameters *pars) {
  this->pars=pars;
}

void PADnoteHarmonicProfile::draw() {
  //
          int ox = x(), oy = y(), lx =w (), ly = h();
          if (!visible()) return;
          float smps[lx];
  
          float realbw = pars->getprofile(smps,lx);
          bool active = active_r();
  
          //draw the equivalent bandwidth
          if (active)
              fl_color(220,220,220);
          else
              fl_color(160,165,165);
          fl_line_style(0);
          int rbw = (int)(realbw * (lx - 1.0) / 2.0);
          for (int i = lx / 2 - rbw; i < (lx / 2 + rbw); i++)
              fl_line(ox+i, oy, ox+i, oy + ly - 1);
  
          fl_line_style(0);
          if (active)
              fl_color(200, 200, 200);
          else
              fl_color(160, 160, 160);
          for (int i = 1; i < 10; i++)
          {
              int kx = (int)(lx / 10.0 * i);
              fl_line(ox + kx, oy, ox + kx, oy + ly - 1);
          }
          for (int i = 1; i < 5; i++)
          {
              int ky = (int)(ly / 5.0 * i);
              fl_line(ox, oy + ly - ky, ox + lx, oy + ly - ky - 1);
          }
  
          fl_color(120, 120, 120);
          fl_line_style(FL_DOT);
          fl_line(ox + lx / 2, oy, ox + lx / 2, oy + ly);
  
          //draw the graph
          fl_line_style(0);
          int old = 0;
          for (int i = 0; i < lx; i++)
          {
              int val = (int) ((ly - 2) * smps[i]);
              if (active)
                  fl_color(180, 210, 240);
              else
                  fl_color(150, 150, 155);
              fl_line(ox + i, oy + ly - 1, ox + i, oy + ly - 1 - val);
              if (active)
                  fl_color(0, 0, 100);
              else
                  fl_color(150, 150, 150);
              if (i > 0)
                  fl_line(ox + i - 1, oy + ly - 2 - old, ox + i, oy + ly - 2 - val);
              old = val;
          }
  
          fl_line_style(FL_DASH);
          if (active)
              fl_color(0, 100, 220);
          else
              fl_color(150, 160, 170);
          fl_line(ox + lx / 2 - rbw, oy, ox + lx / 2 - rbw, oy + ly - 1);
          fl_line(ox + lx / 2 + rbw, oy, ox + lx / 2 + rbw, oy + ly - 1);
          fl_line_style(0);
}

PADnoteOvertonePosition::PADnoteOvertonePosition(int x,int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label) {
  pars=NULL;
}

void PADnoteOvertonePosition::init(PADnoteParameters *pars) {
  synth = pars->getSynthEngine();
  this->pars=pars;
}

void PADnoteOvertonePosition::draw() {
  //
          if (!visible()) return;
          const int maxdb = 60;
          int ox = x(), oy = y(), lx = w(), ly = h();
          const int maxharmonic = 64;
  
          for (int i = 1; i < maxharmonic; i++)
          {
              fl_color(200, 200, 200);
              fl_line_style(FL_DOT);
              if (i % 5 == 0)
                  fl_line_style(0);
              if (i % 10 == 0)
                  fl_color(160, 160, 160);
              int kx = (int)(lx / (float)maxharmonic * i);
              fl_line(ox + kx, oy, ox + kx, oy + ly);
          }
  
          int n = synth->halfoscilsize;
          float spc[n];
          for (int i = 0; i < n; i++)
              spc[i] = 0.0;
          synth->actionLock(lock);
          pars->oscilgen->getspectrum(n, spc, 0);
          synth->actionLock(unlock);
  
          //normalize
          float max = 0;
          for (int i = 0; i < n; i++)
          {
              float x = fabsf(spc[i]);
              if (max < x)
                  max=x;
          }
          if (max < 0.000001)
              max = 1.0;
          max = max * 1.05;
  
          float spectrum[lx];
          for (int i = 0; i < lx; i++)
              spectrum[i] = 0;
  
          for (int i = 1; i < n; i++)
          {
              float nhr = pars->getNhr(i);
              int kx = (int)(lx / (float)maxharmonic * nhr);
              if (kx < 0 || kx > lx)
                  continue;
              spectrum[kx] = spc[i - 1] / max + 1e-9;
          }
  
          fl_color(180, 0, 0);
          fl_line_style(0);
  
          if (pars->Pmode == 2)
          {
              int old = 0;
              for (int i = 1; i < lx; i++)
              {
                  if ((spectrum[i] > 1e-10) || (i == (lx - 1)))
                  {
                      int delta = i - old;
                      float val1 = spectrum[old];
                      float val2 = spectrum[i];
                      float idelta = 1.0 / delta;
                      for (int j = 0; j < delta; j++)
                      {
                          float x = idelta * j;
                          spectrum[old + j] = val1 * (1.0 - x) + val2 * x;
                      }
                      old = i;
                  }
              }
          }
  
          for (int i = 0; i < lx; i++)
          {
              float x = spectrum[i];
              if (x > dB2rap(-maxdb))
                  x = rap2dB(x) / maxdb + 1;
              else
                  continue;
              int yy = (int)(x * ly);
              fl_line(ox+i,oy+ly-1-yy,ox+i,oy+ly-1);
          }
}

void PADnoteUI::cb__i(Fl_Tabs* o, void*) {
  //
            if (o->value() != harmonicstructuregroup)
                applybutton->hide();
            else
                applybutton->show();
}
void PADnoteUI::cb_(Fl_Tabs* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb__i(o,v);
}

void PADnoteUI::cb_hpbasepar1_i(WidgetPDial* o, void*) {
  //
              send_data(64, o->value(), 0xd8);
}
void PADnoteUI::cb_hpbasepar1(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpbasepar1_i(o,v);
}

void PADnoteUI::cb_hpbasetype_i(Fl_Choice* o, void*) {
  //
              send_data(69, o->value(), 0xd0);
}
void PADnoteUI::cb_hpbasetype(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpbasetype_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_hpbasetype[] = {
 {"Gauss", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Square", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"DoubleExp", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_hpfreqmult_i(WidgetPDial* o, void*) {
  //
              send_data(65, o->value(), 0xd8);
}
void PADnoteUI::cb_hpfreqmult(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpfreqmult_i(o,v);
}

void PADnoteUI::cb_hpmpar1_i(WidgetPDial* o, void*) {
  //
              send_data(66, o->value(), 0xd8);
}
void PADnoteUI::cb_hpmpar1(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpmpar1_i(o,v);
}

void PADnoteUI::cb_hpmfreq_i(WidgetPDial* o, void*) {
  //
              send_data(67, o->value(), 0xd8);
}
void PADnoteUI::cb_hpmfreq(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpmfreq_i(o,v);
}

void PADnoteUI::cb_hpamptype_i(Fl_Choice* o, void*) {
  //
                send_data(73, o->value(), 0xd0);
}
void PADnoteUI::cb_hpamptype(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->parent()->user_data()))->cb_hpamptype_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_hpamptype[] = {
 {"OFF", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Gauss", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Sine", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Flat", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_hpampmode_i(Fl_Choice* o, void*) {
  //
                send_data(74, o->value(), 0xd0);
}
void PADnoteUI::cb_hpampmode(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->parent()->user_data()))->cb_hpampmode_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_hpampmode[] = {
 {"Sum", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Mult", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Div1", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Div2", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_hpamppar1_i(WidgetPDial* o, void*) {
  //
                send_data(71, o->value(), 0xd8);
}
void PADnoteUI::cb_hpamppar1(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->parent()->user_data()))->cb_hpamppar1_i(o,v);
}

void PADnoteUI::cb_hpamppar2_i(WidgetPDial* o, void*) {
  //
                send_data(72, o->value(), 0xd8);
}
void PADnoteUI::cb_hpamppar2(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->parent()->user_data()))->cb_hpamppar2_i(o,v);
}

void PADnoteUI::cb_hpautoscale_i(Fl_Check_Button* o, void*) {
  //
              send_data(75, o->value(), 0xd0);
}
void PADnoteUI::cb_hpautoscale(Fl_Check_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpautoscale_i(o,v);
}

void PADnoteUI::cb_hponehalf_i(Fl_Choice* o, void*) {
  //
              send_data(70, o->value(), 0xd0);
}
void PADnoteUI::cb_hponehalf(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hponehalf_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_hponehalf[] = {
 {"Full", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Upper Half", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {"Lower Half", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_hpwidth_i(WidgetPDial* o, void*) {
  //
              send_data(68, o->value(), 0xd8);
}
void PADnoteUI::cb_hpwidth(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hpwidth_i(o,v);
}

void PADnoteUI::cb_Change_i(Fl_Button*, void*) {
  //
            if (oscui!=NULL) delete (oscui);
                oscui = new OscilEditor(pars->oscilgen, osc, cbwidget, applybutton, synth, npart, kititem, 2);
            if ((Fl::event_button() == 3))
                padnotewindow->hide();
}
void PADnoteUI::cb_Change(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_Change_i(o,v);
}

void PADnoteUI::cb_cbwidget_i(Fl_Box*, void*) {
  //
            overtonepos->redraw();
            applybutton->color(FL_RED);
            applybutton->redraw();
}
void PADnoteUI::cb_cbwidget(Fl_Box* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_cbwidget_i(o,v);
}

void PADnoteUI::cb_Resonance_i(Fl_Button*, void*) {
  //
            resui->resonancewindow->redraw();
            resui->Show(false);
            resui->setcbwidget(cbwidget, applybutton);
            if ((Fl::event_button() == 3))
                padnotewindow->hide();
}
void PADnoteUI::cb_Resonance(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_Resonance_i(o,v);
}

void PADnoteUI::cb_bwdial_i(WidgetPDial* o, void*) {
  //
            send_data(16, o->value(), 0xd8);
}
void PADnoteUI::cb_bwdial(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_bwdial_i(o,v);
}

void PADnoteUI::cb_hrpostype_i(Fl_Choice* o, void*) {
  //
                  send_data(51, o->value(), 0xd0);
}
void PADnoteUI::cb_hrpostype(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hrpostype_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_hrpostype[] = {
 {"Harmonic", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"ShiftU", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"ShiftL", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"PowerU", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"PowerL", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Sine", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Power", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_hrpospar1_i(WidgetPDial* o, void*) {
  //
              send_data(48, o->value(), 0xd8);
}
void PADnoteUI::cb_hrpospar1(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hrpospar1_i(o,v);
}

void PADnoteUI::cb_hrpospar2_i(WidgetPDial* o, void*) {
  //
              send_data(49, o->value(), 0xd8);
}
void PADnoteUI::cb_hrpospar2(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hrpospar2_i(o,v);
}

void PADnoteUI::cb_hrpospar3_i(WidgetPDial* o, void*) {
  //
              send_data(50, o->value(), 0xd8);
}
void PADnoteUI::cb_hrpospar3(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hrpospar3_i(o,v);
}

void PADnoteUI::cb_bwscale_i(Fl_Choice* o, void*) {
  //
            send_data(17, o->value(), 0xd0);
}
void PADnoteUI::cb_bwscale(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_bwscale_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_bwscale[] = {
 {"Normal", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"EqualHz", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Quater", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Half", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"75%", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"150%", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Double", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Inv.Half", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_qsamplesize_i(Fl_Choice* o, void*) {
  //
            send_data(83, o->value(), 0xd0);
}
void PADnoteUI::cb_qsamplesize(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_qsamplesize_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_qsamplesize[] = {
 {"16k (Tiny)", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"32k", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"64k (Small)", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"128k", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"256k (Normal)", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"512k", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"1M (Big)", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_qsmpoct_i(Fl_Choice* o, void*) {
  //
            send_data(81, o->value(), 0xd0);
}
void PADnoteUI::cb_qsmpoct(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_qsmpoct_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_qsmpoct[] = {
 {"0.5", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"1", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"2", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"3", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"4", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"6", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"12", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_qoct_i(Fl_Choice* o, void*) {
  //
            send_data(82, o->value(), 0xd0);
}
void PADnoteUI::cb_qoct(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_qoct_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_qoct[] = {
 {"1", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"2", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"3", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"4", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"5", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"6", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"7", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"8", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_qbasenote_i(Fl_Choice* o, void*) {
  //
            send_data(80, o->value(), 0xd0);
}
void PADnoteUI::cb_qbasenote(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_qbasenote_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_qbasenote[] = {
 {"C-2", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"G-2", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"C-3", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"G-3", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"C-4", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"G-4", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"C-5", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"G-5", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {"G-6", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 14, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_spectrummode_i(Fl_Choice* o, void*) {
  //
            send_data(19, o->value(), 0xd0);
}
void PADnoteUI::cb_spectrummode(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->user_data()))->cb_spectrummode_i(o,v);
}

Fl_Menu_Item PADnoteUI::menu_spectrummode[] = {
 {"Bandwidth", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Discrete", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {"Continuous", 0,  0, 0, 0, (uchar)FL_NORMAL_LABEL, 1, 11, 0},
 {0,0,0,0,0,0,0,0,0}
};

void PADnoteUI::cb_octave_i(Fl_Counter* o, void*) {
  //
              send_data(35, o->value(), 0xc8);
}
void PADnoteUI::cb_octave(Fl_Counter* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_octave_i(o,v);
}

void PADnoteUI::cb_coarsedet_i(Fl_Counter* o, void*) {
  //
              send_data(37, o->value(), 0xc0);
}
void PADnoteUI::cb_coarsedet(Fl_Counter* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_coarsedet_i(o,v);
}

void PADnoteUI::cb_detune_i(mwheel_slider_rev* o, void*) {
  //
              send_data(32, o->value(), 0xd8);
}
void PADnoteUI::cb_detune(mwheel_slider_rev* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_detune_i(o,v);
}

void PADnoteUI::cb_detunevalueoutput_i(Fl_Value_Output* o, void*) {
  o->value(getDetune(pars->PDetuneType,0,pars->PDetune));
}
void PADnoteUI::cb_detunevalueoutput(Fl_Value_Output* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_detunevalueoutput_i(o,v);
}

void PADnoteUI::cb_detunetype_i(Fl_Choice* o, void*) {
  //
              send_data(36, o->value(), 0xd0);
}
void PADnoteUI::cb_detunetype(Fl_Choice* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_detunetype_i(o,v);
}

void PADnoteUI::cb_hz440_i(Fl_Check_Button* o, void*) {
  //
              int x = (int) o->value();
              send_data(34, x, 0xd0);
}
void PADnoteUI::cb_hz440(Fl_Check_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_hz440_i(o,v);
}

void PADnoteUI::cb_fixedfreqetdial_i(WidgetPDial* o, void*) {
  //
              send_data(33, o->value(), 0xc8);
}
void PADnoteUI::cb_fixedfreqetdial(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_fixedfreqetdial_i(o,v);
}

void PADnoteUI::cb_bend_i(WidgetPDial* o, void*) {
  //
              send_data(38, o->value(), 0xc8);
}
void PADnoteUI::cb_bend(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_bend_i(o,v);
}

void PADnoteUI::cb_offset_i(WidgetPDial* o, void*) {
  //
              send_data(39, o->value(), 0xc8);
}
void PADnoteUI::cb_offset(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_offset_i(o,v);
}

void PADnoteUI::cb_volume_i(WidgetPDial* o, void*) {
  //
              send_data(0, o->value(), 0xc8);
}
void PADnoteUI::cb_volume(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_volume_i(o,v);
}

void PADnoteUI::cb_vsns_i(WidgetPDial* o, void*) {
  //
              send_data(1, o->value(), 0xc8);
}
void PADnoteUI::cb_vsns(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_vsns_i(o,v);
}

void PADnoteUI::cb_pan_i(WidgetPDial* o, void*) {
  //
              send_data(2, o->value(), 0xd8);
}
void PADnoteUI::cb_pan(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_pan_i(o,v);
}

void PADnoteUI::cb_randompan_i(Fl_Light_Button*, void*) {
  //
              send_data(2, 0, 0xd0);
}
void PADnoteUI::cb_randompan(Fl_Light_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_randompan_i(o,v);
}

void PADnoteUI::cb_dpop_i(WidgetPDial* o, void*) {
  //
              send_data(120, o->value(), 0xc8);
}
void PADnoteUI::cb_dpop(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_dpop_i(o,v);
}

void PADnoteUI::cb_pstr_i(WidgetPDial* o, void*) {
  //
              send_data(121, o->value(), 0xc8);
}
void PADnoteUI::cb_pstr(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_pstr_i(o,v);
}

void PADnoteUI::cb_pt_i(WidgetPDial* o, void*) {
  //
              send_data(122, o->value(), 0xc8);
}
void PADnoteUI::cb_pt(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_pt_i(o,v);
}

void PADnoteUI::cb_pstc_i(WidgetPDial* o, void*) {
  //
              send_data(123, o->value(), 0xc8);
}
void PADnoteUI::cb_pstc(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_pstc_i(o,v);
}

void PADnoteUI::cb_pvel_i(WidgetPDial* o, void*) {
  //
              send_data(124, o->value(), 0xc8);
}
void PADnoteUI::cb_pvel(WidgetPDial* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_pvel_i(o,v);
}

void PADnoteUI::cb_stereo_i(Fl_Check_Button* o, void*) {
  //
              pars->PStereo =(o->value()) ? true : false; hprofile->redraw();
              send_data(112, o->value(), 0xc8);
}
void PADnoteUI::cb_stereo(Fl_Check_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->parent()->parent()->parent()->user_data()))->cb_stereo_i(o,v);
}

void PADnoteUI::cb_applybutton_i(Fl_Button* o, void*) {
  //
        send_data(104, o->value(), 0xd0);
}
void PADnoteUI::cb_applybutton(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb_applybutton_i(o,v);
}

void PADnoteUI::cb_Close_i(Fl_Button*, void*) {
  //
        synth->getGuiMaster()->padNoteX = padnotewindow->x();
        synth->getGuiMaster()->padNoteY = padnotewindow->y();
        padnotewindow->hide();
        if ((Fl::event_button() == 3))
        {
            if (synth->getGuiMaster()->partui->partKitOn)
                synth->getGuiMaster()->partui->instrumentkitlist->show();
            else
                synth->getGuiMaster()->partui->instrumenteditwindow->show();
        };
}
void PADnoteUI::cb_Close(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb_Close_i(o,v);
}

void PADnoteUI::cb_C_i(Fl_Button*, void*) {
  synth->getGuiMaster()->getPresetsUi()->copy(pars);
}
void PADnoteUI::cb_C(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb_C_i(o,v);
}

void PADnoteUI::cb_P_i(Fl_Button*, void*) {
  synth->getGuiMaster()->getPresetsUi()->paste(pars,this);
}
void PADnoteUI::cb_P(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb_P_i(o,v);
}

void PADnoteUI::cb_Export_i(Fl_Button*, void*) {
  //
        char *filename;
        filename=fl_file_chooser("Export samples:","(*.wav)",NULL,0);
        if (filename==NULL)
            return;
        fl_filename_setext(filename,"");
        send_data(94, 0, 0xd0, miscMsgPush(string(filename)));
}
void PADnoteUI::cb_Export(Fl_Button* o, void* v) {
  ((PADnoteUI*)(o->parent()->user_data()))->cb_Export_i(o,v);
}

Fl_Double_Window* PADnoteUI::make_window() {
  { Fl_Double_Window* o = padnotewindow = new Fl_Double_Window(535, 450, "PAD synth Parameters");
    padnotewindow->user_data((void*)(this));
    { Fl_Tabs* o = new Fl_Tabs(0, 0, 535, 405);
      o->callback((Fl_Callback*)cb_);
      { harmonicstructuregroup = new Fl_Group(0, 20, 535, 375, "Harmonic Structure");
        harmonicstructuregroup->box(FL_ENGRAVED_FRAME);
        harmonicstructuregroup->labelsize(12);
        { Fl_Group* o = bwprofilegroup = new Fl_Group(5, 30, 90, 260);
          bwprofilegroup->box(FL_ENGRAVED_FRAME);
          { WidgetPDial* o = hpbasepar1 = new WidgetPDial(20, 75, 25, 25, "Width");
            hpbasepar1->box(FL_ROUND_UP_BOX);
            hpbasepar1->color(FL_BACKGROUND_COLOR);
            hpbasepar1->selection_color(FL_INACTIVE_COLOR);
            hpbasepar1->labeltype(FL_NORMAL_LABEL);
            hpbasepar1->labelfont(0);
            hpbasepar1->labelsize(10);
            hpbasepar1->labelcolor(FL_FOREGROUND_COLOR);
            hpbasepar1->maximum(127);
            hpbasepar1->step(1);
            hpbasepar1->callback((Fl_Callback*)cb_hpbasepar1);
            hpbasepar1->align(Fl_Align(FL_ALIGN_TOP));
            hpbasepar1->when(FL_WHEN_CHANGED);
            o->value(pars->Php.base.par1);
            o->init(80);
          } // WidgetPDial* hpbasepar1
          { Fl_Choice* o = hpbasetype = new Fl_Choice(15, 45, 75, 15, "Base Type");
            hpbasetype->down_box(FL_BORDER_BOX);
            hpbasetype->labelsize(10);
            hpbasetype->textsize(10);
            hpbasetype->callback((Fl_Callback*)cb_hpbasetype);
            hpbasetype->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            hpbasetype->menu(menu_hpbasetype);
            o->value(pars->Php.base.type);
          } // Fl_Choice* hpbasetype
          { WidgetPDial* o = hpfreqmult = new WidgetPDial(55, 75, 25, 25, "FreqMlt");
            hpfreqmult->box(FL_ROUND_UP_BOX);
            hpfreqmult->color(FL_BACKGROUND_COLOR);
            hpfreqmult->selection_color(FL_INACTIVE_COLOR);
            hpfreqmult->labeltype(FL_NORMAL_LABEL);
            hpfreqmult->labelfont(0);
            hpfreqmult->labelsize(10);
            hpfreqmult->labelcolor(FL_FOREGROUND_COLOR);
            hpfreqmult->maximum(127);
            hpfreqmult->step(1);
            hpfreqmult->callback((Fl_Callback*)cb_hpfreqmult);
            hpfreqmult->align(Fl_Align(FL_ALIGN_TOP));
            hpfreqmult->when(FL_WHEN_CHANGED);
            o->value(pars->Php.freqmult);
            o->init(8);
          } // WidgetPDial* hpfreqmult
          { WidgetPDial* o = hpmpar1 = new WidgetPDial(15, 115, 20, 20, "Str");
            hpmpar1->box(FL_ROUND_UP_BOX);
            hpmpar1->color(FL_BACKGROUND_COLOR);
            hpmpar1->selection_color(FL_INACTIVE_COLOR);
            hpmpar1->labeltype(FL_NORMAL_LABEL);
            hpmpar1->labelfont(0);
            hpmpar1->labelsize(10);
            hpmpar1->labelcolor(FL_FOREGROUND_COLOR);
            hpmpar1->maximum(127);
            hpmpar1->step(1);
            hpmpar1->callback((Fl_Callback*)cb_hpmpar1);
            hpmpar1->align(Fl_Align(FL_ALIGN_TOP));
            hpmpar1->when(FL_WHEN_CHANGED);
            o->value(pars->Php.modulator.par1);
            o->init(0);
          } // WidgetPDial* hpmpar1
          { WidgetPDial* o = hpmfreq = new WidgetPDial(40, 115, 20, 20, "SFreq");
            hpmfreq->box(FL_ROUND_UP_BOX);
            hpmfreq->color(FL_BACKGROUND_COLOR);
            hpmfreq->selection_color(FL_INACTIVE_COLOR);
            hpmfreq->labeltype(FL_NORMAL_LABEL);
            hpmfreq->labelfont(0);
            hpmfreq->labelsize(10);
            hpmfreq->labelcolor(FL_FOREGROUND_COLOR);
            hpmfreq->maximum(127);
            hpmfreq->step(1);
            hpmfreq->callback((Fl_Callback*)cb_hpmfreq);
            hpmfreq->align(Fl_Align(FL_ALIGN_TOP));
            hpmfreq->when(FL_WHEN_CHANGED);
            o->value(pars->Php.modulator.freq);
            o->init(30);
          } // WidgetPDial* hpmfreq
          { Fl_Group* o = new Fl_Group(10, 160, 80, 105);
            o->box(FL_BORDER_BOX);
            { Fl_Choice* o = hpamptype = new Fl_Choice(15, 175, 70, 15, "AmpMultiplier");
              hpamptype->down_box(FL_BORDER_BOX);
              hpamptype->labelsize(10);
              hpamptype->textsize(10);
              hpamptype->callback((Fl_Callback*)cb_hpamptype);
              hpamptype->align(Fl_Align(FL_ALIGN_TOP_LEFT));
              hpamptype->menu(menu_hpamptype);
              o->value(pars->Php.amp.type);
            } // Fl_Choice* hpamptype
            { Fl_Choice* o = hpampmode = new Fl_Choice(15, 205, 70, 15, "AmpMode");
              hpampmode->down_box(FL_BORDER_BOX);
              hpampmode->labelsize(10);
              hpampmode->textsize(10);
              hpampmode->callback((Fl_Callback*)cb_hpampmode);
              hpampmode->align(Fl_Align(FL_ALIGN_TOP_LEFT));
              hpampmode->menu(menu_hpampmode);
              o->value(pars->Php.amp.mode);
            } // Fl_Choice* hpampmode
            { WidgetPDial* o = hpamppar1 = new WidgetPDial(15, 235, 25, 25, "Par1");
              hpamppar1->box(FL_ROUND_UP_BOX);
              hpamppar1->color(FL_BACKGROUND_COLOR);
              hpamppar1->selection_color(FL_INACTIVE_COLOR);
              hpamppar1->labeltype(FL_NORMAL_LABEL);
              hpamppar1->labelfont(0);
              hpamppar1->labelsize(10);
              hpamppar1->labelcolor(FL_FOREGROUND_COLOR);
              hpamppar1->maximum(127);
              hpamppar1->step(1);
              hpamppar1->callback((Fl_Callback*)cb_hpamppar1);
              hpamppar1->align(Fl_Align(FL_ALIGN_TOP));
              hpamppar1->when(FL_WHEN_CHANGED);
              o->value(pars->Php.amp.par1);
              o->init(80);
            } // WidgetPDial* hpamppar1
            { WidgetPDial* o = hpamppar2 = new WidgetPDial(55, 235, 25, 25, "Par2");
              hpamppar2->box(FL_ROUND_UP_BOX);
              hpamppar2->color(FL_BACKGROUND_COLOR);
              hpamppar2->selection_color(FL_INACTIVE_COLOR);
              hpamppar2->labeltype(FL_NORMAL_LABEL);
              hpamppar2->labelfont(0);
              hpamppar2->labelsize(10);
              hpamppar2->labelcolor(FL_FOREGROUND_COLOR);
              hpamppar2->maximum(127);
              hpamppar2->step(1);
              hpamppar2->callback((Fl_Callback*)cb_hpamppar2);
              hpamppar2->align(Fl_Align(FL_ALIGN_TOP));
              hpamppar2->when(FL_WHEN_CHANGED);
              o->value(pars->Php.amp.par2);
              o->init(64);
            } // WidgetPDial* hpamppar2
            o->end();
          } // Fl_Group* o
          { Fl_Check_Button* o = hpautoscale = new Fl_Check_Button(10, 270, 60, 15, "autoscale");
            hpautoscale->down_box(FL_DOWN_BOX);
            hpautoscale->labelsize(10);
            hpautoscale->callback((Fl_Callback*)cb_hpautoscale);
            o->value(pars->Php.autoscale);
          } // Fl_Check_Button* hpautoscale
          { Fl_Choice* o = hponehalf = new Fl_Choice(10, 143, 80, 15);
            hponehalf->down_box(FL_BORDER_BOX);
            hponehalf->labelsize(10);
            hponehalf->textsize(10);
            hponehalf->callback((Fl_Callback*)cb_hponehalf);
            hponehalf->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            hponehalf->menu(menu_hponehalf);
            o->value(pars->Php.onehalf);
          } // Fl_Choice* hponehalf
          { WidgetPDial* o = hpwidth = new WidgetPDial(65, 115, 20, 20, "Size");
            hpwidth->box(FL_ROUND_UP_BOX);
            hpwidth->color(FL_BACKGROUND_COLOR);
            hpwidth->selection_color(FL_INACTIVE_COLOR);
            hpwidth->labeltype(FL_NORMAL_LABEL);
            hpwidth->labelfont(0);
            hpwidth->labelsize(10);
            hpwidth->labelcolor(FL_FOREGROUND_COLOR);
            hpwidth->maximum(127);
            hpwidth->step(1);
            hpwidth->callback((Fl_Callback*)cb_hpwidth);
            hpwidth->align(Fl_Align(FL_ALIGN_TOP));
            hpwidth->when(FL_WHEN_CHANGED);
            o->value(pars->Php.width);
            o->init(127);
          } // WidgetPDial* hpwidth
          if (pars->Pmode!=0) o->deactivate();
          bwprofilegroup->end();
        } // Fl_Group* bwprofilegroup
        { Fl_Group* o = new Fl_Group(100, 155, 270, 135);
          o->box(FL_THIN_DOWN_BOX);
          o->color(FL_GRAY0);
          o->selection_color((Fl_Color)71);
          o->labelcolor((Fl_Color)179);
          o->align(Fl_Align(FL_ALIGN_BOTTOM_LEFT));
          osc=new Oscilloscope(o->x(),o->y(),o->w(),o->h(),"");
          osc->init(pars->oscilgen, synth);
          o->end();
        } // Fl_Group* o
        { Fl_Button* o = new Fl_Button(375, 270, 60, 20, "Change");
          o->box(FL_THIN_UP_BOX);
          o->labelfont(1);
          o->labelsize(11);
          o->callback((Fl_Callback*)cb_Change);
        } // Fl_Button* o
        { cbwidget = new Fl_Box(125, 135, 205, 20, "Harmonic Content");
          cbwidget->labelsize(12);
          cbwidget->callback((Fl_Callback*)cb_cbwidget);
          cbwidget->align(Fl_Align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE));
        } // Fl_Box* cbwidget
        { Fl_Button* o = new Fl_Button(375, 225, 80, 20, "Resonance");
          o->box(FL_THIN_UP_BOX);
          o->labelsize(12);
          o->callback((Fl_Callback*)cb_Resonance);
        } // Fl_Button* o
        { WidgetPDial* o = bwdial = new WidgetPDial(15, 295, 35, 35, "BandWidth");
          bwdial->box(FL_ROUND_UP_BOX);
          bwdial->color(FL_BACKGROUND_COLOR);
          bwdial->selection_color(FL_INACTIVE_COLOR);
          bwdial->labeltype(FL_NORMAL_LABEL);
          bwdial->labelfont(0);
          bwdial->labelsize(10);
          bwdial->labelcolor(FL_FOREGROUND_COLOR);
          bwdial->maximum(1000);
          bwdial->step(1);
          bwdial->callback((Fl_Callback*)cb_bwdial);
          bwdial->align(Fl_Align(FL_ALIGN_BOTTOM));
          bwdial->when(FL_WHEN_CHANGED);
          o->value(pars->Pbandwidth);
          if (pars->Pmode!=0) o->deactivate();
          o->init(500);
          o->setValueType(VC_BandWidth);
        } // WidgetPDial* bwdial
        { Fl_Value_Output* o = bwcents = new Fl_Value_Output(55, 305, 55, 15, "cents");
          bwcents->labelsize(10);
          bwcents->maximum(10000);
          bwcents->align(Fl_Align(FL_ALIGN_BOTTOM_LEFT));
          o->step(0.1, 1);
          o->value(pars->setPbandwidth(pars->Pbandwidth));
          if (pars->Pmode!=0) o->deactivate();
        } // Fl_Value_Output* bwcents
        { Fl_Group* o = new Fl_Group(315, 295, 215, 45);
          o->box(FL_ENGRAVED_FRAME);
          { Fl_Choice* o = hrpostype = new Fl_Choice(325, 310, 80, 20, "OvertonesPosition");
            hrpostype->down_box(FL_BORDER_BOX);
            hrpostype->labelsize(10);
            hrpostype->textsize(11);
            hrpostype->callback((Fl_Callback*)cb_hrpostype);
            hrpostype->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            hrpostype->menu(menu_hrpostype);
            o->value(pars->Phrpos.type);
          } // Fl_Choice* hrpostype
          { WidgetPDial* o = hrpospar1 = new WidgetPDial(425, 310, 25, 25, "Par1");
            hrpospar1->box(FL_ROUND_UP_BOX);
            hrpospar1->color(FL_BACKGROUND_COLOR);
            hrpospar1->selection_color(FL_INACTIVE_COLOR);
            hrpospar1->labeltype(FL_NORMAL_LABEL);
            hrpospar1->labelfont(0);
            hrpospar1->labelsize(10);
            hrpospar1->labelcolor(FL_FOREGROUND_COLOR);
            hrpospar1->maximum(255);
            hrpospar1->step(1);
            hrpospar1->callback((Fl_Callback*)cb_hrpospar1);
            hrpospar1->align(Fl_Align(FL_ALIGN_TOP));
            hrpospar1->when(FL_WHEN_CHANGED);
            o->value(pars->Phrpos.par1);
            o->init(64);
          } // WidgetPDial* hrpospar1
          { WidgetPDial* o = hrpospar2 = new WidgetPDial(460, 310, 25, 25, "Par2");
            hrpospar2->box(FL_ROUND_UP_BOX);
            hrpospar2->color(FL_BACKGROUND_COLOR);
            hrpospar2->selection_color(FL_INACTIVE_COLOR);
            hrpospar2->labeltype(FL_NORMAL_LABEL);
            hrpospar2->labelfont(0);
            hrpospar2->labelsize(10);
            hrpospar2->labelcolor(FL_FOREGROUND_COLOR);
            hrpospar2->maximum(255);
            hrpospar2->step(1);
            hrpospar2->callback((Fl_Callback*)cb_hrpospar2);
            hrpospar2->align(Fl_Align(FL_ALIGN_TOP));
            hrpospar2->when(FL_WHEN_CHANGED);
            o->value(pars->Phrpos.par2);
            o->init(64);
          } // WidgetPDial* hrpospar2
          { WidgetPDial* o = hrpospar3 = new WidgetPDial(495, 310, 25, 25, "ForceH");
            hrpospar3->box(FL_ROUND_UP_BOX);
            hrpospar3->color(FL_BACKGROUND_COLOR);
            hrpospar3->selection_color(FL_INACTIVE_COLOR);
            hrpospar3->labeltype(FL_NORMAL_LABEL);
            hrpospar3->labelfont(0);
            hrpospar3->labelsize(10);
            hrpospar3->labelcolor(FL_FOREGROUND_COLOR);
            hrpospar3->maximum(255);
            hrpospar3->step(1);
            hrpospar3->callback((Fl_Callback*)cb_hrpospar3);
            hrpospar3->align(Fl_Align(FL_ALIGN_TOP));
            hrpospar3->when(FL_WHEN_CHANGED);
            o->value(pars->Phrpos.par3);
            o->init(0);
            o->setValueType(VC_percent255);
          } // WidgetPDial* hrpospar3
          o->end();
        } // Fl_Group* o
        { Fl_Choice* o = bwscale = new Fl_Choice(120, 305, 80, 20, "Bandwidth Scale");
          bwscale->down_box(FL_BORDER_BOX);
          bwscale->labelsize(10);
          bwscale->textsize(11);
          bwscale->callback((Fl_Callback*)cb_bwscale);
          bwscale->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          bwscale->menu(menu_bwscale);
          o->value(pars->Pbwscale);
          if (pars->Pmode!=0) o->deactivate();
        } // Fl_Choice* bwscale
        { Fl_Group* o = overtonepos = new Fl_Group(5, 345, 525, 45);
          overtonepos->box(FL_FLAT_BOX);
          overtonepos->color(FL_LIGHT3);
          overtonepos->selection_color((Fl_Color)218);
          overtonepos->labelcolor(FL_GREEN);
          PADnoteOvertonePosition *opui=new PADnoteOvertonePosition(o->x(),o->y(),o->w(),o->h(),"");
          opui->init(pars);
          overtonepos->end();
        } // Fl_Group* overtonepos
        { Fl_Choice* o = qsamplesize = new Fl_Choice(375, 190, 115, 20, "Sample Size");
          qsamplesize->down_box(FL_BORDER_BOX);
          qsamplesize->labelsize(10);
          qsamplesize->textsize(11);
          qsamplesize->callback((Fl_Callback*)cb_qsamplesize);
          qsamplesize->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          qsamplesize->menu(menu_qsamplesize);
          o->value(pars->Pquality.samplesize);
        } // Fl_Choice* qsamplesize
        { Fl_Choice* o = qsmpoct = new Fl_Choice(430, 155, 45, 20, "smp/oct");
          qsmpoct->down_box(FL_BORDER_BOX);
          qsmpoct->labelsize(11);
          qsmpoct->textsize(11);
          qsmpoct->callback((Fl_Callback*)cb_qsmpoct);
          qsmpoct->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          qsmpoct->menu(menu_qsmpoct);
          o->value(pars->Pquality.smpoct);
        } // Fl_Choice* qsmpoct
        { Fl_Choice* o = qoct = new Fl_Choice(480, 155, 45, 20, "no.oct");
          qoct->down_box(FL_BORDER_BOX);
          qoct->labelsize(11);
          qoct->textsize(11);
          qoct->callback((Fl_Callback*)cb_qoct);
          qoct->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          qoct->menu(menu_qoct);
          o->value(pars->Pquality.oct);
        } // Fl_Choice* qoct
        { Fl_Choice* o = qbasenote = new Fl_Choice(375, 155, 50, 20, "base");
          qbasenote->down_box(FL_BORDER_BOX);
          qbasenote->labelsize(11);
          qbasenote->textsize(11);
          qbasenote->callback((Fl_Callback*)cb_qbasenote);
          qbasenote->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          qbasenote->menu(menu_qbasenote);
          o->value(pars->Pquality.basenote);
        } // Fl_Choice* qbasenote
        { Fl_Group* o = hprofile = new Fl_Group(100, 45, 430, 90);
          hprofile->box(FL_FLAT_BOX);
          hprofile->color(FL_LIGHT3);
          hprofile->selection_color((Fl_Color)218);
          hprofile->labelcolor(FL_GREEN);
          PADnoteHarmonicProfile *hpui=new PADnoteHarmonicProfile(o->x(),o->y(),o->w(),o->h(),"");
          hpui->init(pars);
          if (pars->Pmode!=0) { o->deactivate(); o->color(48);};
          hprofile->end();
        } // Fl_Group* hprofile
        { Fl_Box* o = new Fl_Box(160, 25, 315, 20, "Profile of One Harmonic (Frequency Distribution)");
          o->labelsize(12);
        } // Fl_Box* o
        { Fl_Choice* o = spectrummode = new Fl_Choice(220, 305, 90, 20, "Spectrum Mode");
          spectrummode->down_box(FL_BORDER_BOX);
          spectrummode->labelfont(1);
          spectrummode->labelsize(10);
          spectrummode->textsize(11);
          spectrummode->callback((Fl_Callback*)cb_spectrummode);
          spectrummode->align(Fl_Align(FL_ALIGN_TOP_LEFT));
          spectrummode->menu(menu_spectrummode);
          o->value(pars->Pmode);
        } // Fl_Choice* spectrummode
        harmonicstructuregroup->end();
      } // Fl_Group* harmonicstructuregroup
      { Fl_Group* o = new Fl_Group(0, 20, 535, 385, "Envelopes&LFOs");
        o->box(FL_ENGRAVED_BOX);
        o->labelsize(12);
        o->hide();
        { Fl_Group* o = new Fl_Group(5, 275, 525, 125, "FREQUENCY");
          o->box(FL_THIN_UP_BOX);
          o->labelfont(1);
          o->labelsize(12);
          o->align(Fl_Align(FL_ALIGN_TOP|FL_ALIGN_INSIDE));
          { EnvelopeUI* o = freqenv = new EnvelopeUI(10, 325, 205, 70, "PADSynth - Frequency Envelope");
            freqenv->box(FL_FLAT_BOX);
            freqenv->color((Fl_Color)51);
            freqenv->selection_color(FL_BACKGROUND_COLOR);
            freqenv->labeltype(FL_NORMAL_LABEL);
            freqenv->labelfont(0);
            freqenv->labelsize(12);
            freqenv->labelcolor(FL_FOREGROUND_COLOR);
            freqenv->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            freqenv->when(FL_WHEN_RELEASE);
            o->init(pars->FreqEnvelope, npart, kititem, 2, 1);
            freqenv->end();
          } // EnvelopeUI* freqenv
          { Fl_Counter* o = octave = new Fl_Counter(465, 305, 45, 15, "Octave");
            octave->tooltip("Octave");
            octave->type(1);
            octave->color((Fl_Color)237);
            octave->selection_color(FL_FOREGROUND_COLOR);
            octave->labelsize(10);
            octave->minimum(-8);
            octave->maximum(7);
            octave->step(1);
            octave->textfont(1);
            octave->textsize(11);
            octave->callback((Fl_Callback*)cb_octave);
            octave->align(Fl_Align(FL_ALIGN_TOP));
            int k=pars->PCoarseDetune/1024;
            if (k>=8) k-=16;
            o->value(k);
          } // Fl_Counter* octave
          { Fl_Counter* o = coarsedet = new Fl_Counter(455, 375, 60, 20, "Coarse det.");
            coarsedet->tooltip("Coarse Detune");
            coarsedet->labelsize(10);
            coarsedet->minimum(-64);
            coarsedet->maximum(63);
            coarsedet->step(1);
            coarsedet->textfont(1);
            coarsedet->textsize(11);
            coarsedet->callback((Fl_Callback*)cb_coarsedet);
            coarsedet->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            int k=pars->PCoarseDetune%1024;
            if (k>=512) k-=1024;
            o->value(k);
            o->lstep(10);
          } // Fl_Counter* coarsedet
          { LFOUI* o = freqlfo = new LFOUI(215, 325, 230, 70, "Frequency LFO     ");
            freqlfo->box(FL_FLAT_BOX);
            freqlfo->color(FL_DARK1);
            freqlfo->selection_color(FL_BACKGROUND_COLOR);
            freqlfo->labeltype(FL_NORMAL_LABEL);
            freqlfo->labelfont(0);
            freqlfo->labelsize(12);
            freqlfo->labelcolor(FL_FOREGROUND_COLOR);
            freqlfo->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            freqlfo->when(FL_WHEN_RELEASE);
            o->init(pars->FreqLfo, npart, kititem, 2, 1);
            freqlfo->end();
          } // LFOUI* freqlfo
          { mwheel_slider_rev* o = detune = new mwheel_slider_rev(57, 305, 388, 15);
            detune->tooltip("Fine Detune (cents)");
            detune->type(5);
            detune->box(FL_THIN_DOWN_BOX);
            detune->color(FL_BACKGROUND_COLOR);
            detune->selection_color(FL_BACKGROUND_COLOR);
            detune->labeltype(FL_NORMAL_LABEL);
            detune->labelfont(0);
            detune->labelsize(14);
            detune->labelcolor(FL_FOREGROUND_COLOR);
            detune->minimum(-8192);
            detune->maximum(8191);
            detune->step(1);
            detune->callback((Fl_Callback*)cb_detune);
            detune->align(Fl_Align(FL_ALIGN_BOTTOM));
            detune->when(FL_WHEN_CHANGED);
            o->value(pars->PDetune-8192);
          } // mwheel_slider_rev* detune
          { Fl_Value_Output* o = detunevalueoutput = new Fl_Value_Output(12, 305, 45, 15, "Detune");
            detunevalueoutput->labelsize(10);
            detunevalueoutput->minimum(-5000);
            detunevalueoutput->maximum(5000);
            detunevalueoutput->textfont(1);
            detunevalueoutput->textsize(10);
            detunevalueoutput->callback((Fl_Callback*)cb_detunevalueoutput);
            detunevalueoutput->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            o->step(0.01, 1);
            o->value(getDetune(pars->PDetuneType,0,pars->PDetune));
          } // Fl_Value_Output* detunevalueoutput
          { Fl_Choice* o = detunetype = new Fl_Choice(450, 345, 75, 15, "Detune Type");
            detunetype->down_box(FL_BORDER_BOX);
            detunetype->labelsize(10);
            detunetype->textfont(1);
            detunetype->textsize(10);
            detunetype->callback((Fl_Callback*)cb_detunetype);
            detunetype->align(Fl_Align(FL_ALIGN_TOP_LEFT));
            o->add("L35cents");o->add("L10cents");o->add("E100cents");o->add("E1200cents");
            o->value(pars->PDetuneType-1);
          } // Fl_Choice* detunetype
          { Fl_Check_Button* o = hz440 = new Fl_Check_Button(365, 284, 50, 15, "440Hz");
            hz440->tooltip("set the base frequency to 440Hz");
            hz440->down_box(FL_DOWN_BOX);
            hz440->labelfont(1);
            hz440->labelsize(10);
            hz440->callback((Fl_Callback*)cb_hz440);
            o->value(pars->Pfixedfreq);
          } // Fl_Check_Button* hz440
          { WidgetPDial* o = fixedfreqetdial = new WidgetPDial(420, 280, 20, 20, "Eq.T.");
            fixedfreqetdial->tooltip("How the frequency varies according to the keyboard (leftmost for fixed freque\
ncy)");
            fixedfreqetdial->box(FL_ROUND_UP_BOX);
            fixedfreqetdial->color(FL_BACKGROUND_COLOR);
            fixedfreqetdial->selection_color(FL_INACTIVE_COLOR);
            fixedfreqetdial->labeltype(FL_NORMAL_LABEL);
            fixedfreqetdial->labelfont(0);
            fixedfreqetdial->labelsize(10);
            fixedfreqetdial->labelcolor(FL_FOREGROUND_COLOR);
            fixedfreqetdial->maximum(127);
            fixedfreqetdial->step(1);
            fixedfreqetdial->callback((Fl_Callback*)cb_fixedfreqetdial);
            fixedfreqetdial->align(Fl_Align(FL_ALIGN_RIGHT));
            fixedfreqetdial->when(FL_WHEN_CHANGED);
            o->value(pars->PfixedfreqET);
            if (pars->Pfixedfreq==0) o->deactivate();
            o->init(0);
            o->setValueType(VC_FixedFreqET);
          } // WidgetPDial* fixedfreqetdial
          { WidgetPDial* o = bend = new WidgetPDial(80, 280, 20, 20, "Bend");
            bend->tooltip("Pitch bend range scaling");
            bend->box(FL_ROUND_UP_BOX);
            bend->color(FL_BACKGROUND_COLOR);
            bend->selection_color(FL_INACTIVE_COLOR);
            bend->labeltype(FL_NORMAL_LABEL);
            bend->labelfont(0);
            bend->labelsize(10);
            bend->labelcolor(FL_FOREGROUND_COLOR);
            bend->maximum(127);
            bend->step(1);
            bend->callback((Fl_Callback*)cb_bend);
            bend->align(Fl_Align(FL_ALIGN_RIGHT));
            bend->when(FL_WHEN_CHANGED);
            o->value(pars->PBendAdjust);
            o->init(88);
            o->setValueType(VC_PitchBend);
          } // WidgetPDial* bend
          { WidgetPDial* o = offset = new WidgetPDial(140, 280, 20, 20, "Offset");
            offset->tooltip("Offset of the frequency in Hz.");
            offset->box(FL_ROUND_UP_BOX);
            offset->color(FL_BACKGROUND_COLOR);
            offset->selection_color(FL_INACTIVE_COLOR);
            offset->labeltype(FL_NORMAL_LABEL);
            offset->labelfont(0);
            offset->labelsize(10);
            offset->labelcolor(FL_FOREGROUND_COLOR);
            offset->maximum(127);
            offset->step(1);
            offset->callback((Fl_Callback*)cb_offset);
            offset->align(Fl_Align(FL_ALIGN_RIGHT));
            offset->when(FL_WHEN_CHANGED);
            o->value(pars->POffsetHz);
            o->init(64);
            o->setValueType(VC_FreqOffsetHz);
          } // WidgetPDial* offset
          o->end();
        } // Fl_Group* o
        { Fl_Group* o = new Fl_Group(5, 25, 240, 250, "AMPLITUDE");
          o->box(FL_THIN_UP_FRAME);
          o->labelfont(1);
          o->labelsize(12);
          o->align(Fl_Align(FL_ALIGN_TOP|FL_ALIGN_INSIDE));
          { WidgetPDial* o = volume = new WidgetPDial(24, 42, 34, 34, "Volume");
            volume->tooltip("Volume");
            volume->box(FL_ROUND_UP_BOX);
            volume->color(FL_BACKGROUND_COLOR);
            volume->selection_color(FL_INACTIVE_COLOR);
            volume->labeltype(FL_NORMAL_LABEL);
            volume->labelfont(0);
            volume->labelsize(10);
            volume->labelcolor(FL_FOREGROUND_COLOR);
            volume->maximum(127);
            volume->step(1);
            volume->callback((Fl_Callback*)cb_volume);
            volume->align(Fl_Align(FL_ALIGN_BOTTOM));
            volume->when(FL_WHEN_CHANGED);
            o->setValueType(VC_InstrumentVolume);
            o->value(pars->PVolume);
            o->init(90);
          } // WidgetPDial* volume
          { WidgetPDial* o = vsns = new WidgetPDial(102, 42, 34, 34, "Vel Sens");
            vsns->tooltip("Velocity Sensing Function (rightmost to disable)");
            vsns->box(FL_ROUND_UP_BOX);
            vsns->color(FL_BACKGROUND_COLOR);
            vsns->selection_color(FL_INACTIVE_COLOR);
            vsns->labeltype(FL_NORMAL_LABEL);
            vsns->labelfont(0);
            vsns->labelsize(10);
            vsns->labelcolor(FL_FOREGROUND_COLOR);
            vsns->maximum(127);
            vsns->step(1);
            vsns->callback((Fl_Callback*)cb_vsns);
            vsns->align(Fl_Align(FL_ALIGN_BOTTOM));
            vsns->when(FL_WHEN_CHANGED);
            o->value(pars->PAmpVelocityScaleFunction);
            o->init(64);
            o->setValueType(VC_AmpVelocitySense);
          } // WidgetPDial* vsns
          { WidgetPDial* o = pan = new WidgetPDial(175, 42, 34, 34, "Pan");
            pan->tooltip("Panning - leftmost/zero is Random");
            pan->box(FL_ROUND_UP_BOX);
            pan->color(FL_BACKGROUND_COLOR);
            pan->selection_color(FL_INACTIVE_COLOR);
            pan->labeltype(FL_NORMAL_LABEL);
            pan->labelfont(0);
            pan->labelsize(10);
            pan->labelcolor(FL_FOREGROUND_COLOR);
            pan->maximum(127);
            pan->step(1);
            pan->callback((Fl_Callback*)cb_pan);
            pan->align(Fl_Align(FL_ALIGN_BOTTOM_LEFT));
            pan->when(FL_WHEN_CHANGED);
            o->setValueType(VC_PanningRandom);
            o->value(pars->PPanning);
            o->init(64);
          } // WidgetPDial* pan
          { Fl_Light_Button* o = randompan = new Fl_Light_Button(197, 77, 7, 13, "Rand");
            randompan->tooltip("Random panning On/Off");
            randompan->box(FL_NO_BOX);
            randompan->color((Fl_Color)35);
            randompan->selection_color(FL_RED);
            randompan->labelsize(10);
            randompan->callback((Fl_Callback*)cb_randompan);
            randompan->align(Fl_Align(72));
            o->value(pars->randomPan());
          } // Fl_Light_Button* randompan
          { WidgetPDial* o = dpop = new WidgetPDial(85, 247, 25, 25, "D.Pop");
            dpop->tooltip("Pop Suppression");
            dpop->box(FL_ROUND_UP_BOX);
            dpop->color(FL_BACKGROUND_COLOR);
            dpop->selection_color(FL_INACTIVE_COLOR);
            dpop->labeltype(FL_NORMAL_LABEL);
            dpop->labelfont(0);
            dpop->labelsize(10);
            dpop->labelcolor(FL_FOREGROUND_COLOR);
            dpop->maximum(127);
            dpop->step(1);
            dpop->callback((Fl_Callback*)cb_dpop);
            dpop->align(Fl_Align(FL_ALIGN_TOP));
            dpop->when(FL_WHEN_CHANGED);
            o->value(pars->Fadein_adjustment);
            o->init(20);
          } // WidgetPDial* dpop
          { WidgetPDial* o = pstr = new WidgetPDial(125, 247, 25, 25, "P.Str.");
            pstr->tooltip("Punch Strength");
            pstr->box(FL_ROUND_UP_BOX);
            pstr->color(FL_BACKGROUND_COLOR);
            pstr->selection_color(FL_INACTIVE_COLOR);
            pstr->labeltype(FL_NORMAL_LABEL);
            pstr->labelfont(0);
            pstr->labelsize(10);
            pstr->labelcolor(FL_FOREGROUND_COLOR);
            pstr->maximum(127);
            pstr->step(1);
            pstr->callback((Fl_Callback*)cb_pstr);
            pstr->align(Fl_Align(FL_ALIGN_TOP));
            pstr->when(FL_WHEN_CHANGED);
            o->value(pars->PPunchStrength);
            o->init(0);
          } // WidgetPDial* pstr
          { WidgetPDial* o = pt = new WidgetPDial(155, 247, 25, 25, "P.t.");
            pt->tooltip("Punch Time (duration)");
            pt->box(FL_ROUND_UP_BOX);
            pt->color(FL_BACKGROUND_COLOR);
            pt->selection_color(FL_INACTIVE_COLOR);
            pt->labeltype(FL_NORMAL_LABEL);
            pt->labelfont(0);
            pt->labelsize(10);
            pt->labelcolor(FL_FOREGROUND_COLOR);
            pt->maximum(127);
            pt->step(1);
            pt->callback((Fl_Callback*)cb_pt);
            pt->align(Fl_Align(FL_ALIGN_TOP));
            pt->when(FL_WHEN_CHANGED);
            o->value(pars->PPunchTime);
            o->init(60);
          } // WidgetPDial* pt
          { WidgetPDial* o = pstc = new WidgetPDial(185, 247, 25, 25, "P.Stc.");
            pstc->tooltip("Punch Stretch");
            pstc->box(FL_ROUND_UP_BOX);
            pstc->color(FL_BACKGROUND_COLOR);
            pstc->selection_color(FL_INACTIVE_COLOR);
            pstc->labeltype(FL_NORMAL_LABEL);
            pstc->labelfont(0);
            pstc->labelsize(10);
            pstc->labelcolor(FL_FOREGROUND_COLOR);
            pstc->maximum(127);
            pstc->step(1);
            pstc->callback((Fl_Callback*)cb_pstc);
            pstc->align(Fl_Align(FL_ALIGN_TOP));
            pstc->when(FL_WHEN_CHANGED);
            o->value(pars->PPunchStretch);
            o->init(64);
          } // WidgetPDial* pstc
          { WidgetPDial* o = pvel = new WidgetPDial(215, 247, 25, 25, "P.Vel.");
            pvel->tooltip("Punch Velocity Sensing");
            pvel->box(FL_ROUND_UP_BOX);
            pvel->color(FL_BACKGROUND_COLOR);
            pvel->selection_color(FL_INACTIVE_COLOR);
            pvel->labeltype(FL_NORMAL_LABEL);
            pvel->labelfont(0);
            pvel->labelsize(10);
            pvel->labelcolor(FL_FOREGROUND_COLOR);
            pvel->maximum(127);
            pvel->step(1);
            pvel->callback((Fl_Callback*)cb_pvel);
            pvel->align(Fl_Align(FL_ALIGN_TOP));
            pvel->when(FL_WHEN_CHANGED);
            o->value(pars->PPunchVelocitySensing);
            o->init(72);
          } // WidgetPDial* pvel
          { EnvelopeUI* o = ampenv = new EnvelopeUI(10, 95, 205, 70, "PADSynth - Amplitude Envelope");
            ampenv->box(FL_FLAT_BOX);
            ampenv->color((Fl_Color)51);
            ampenv->selection_color(FL_BACKGROUND_COLOR);
            ampenv->labeltype(FL_NORMAL_LABEL);
            ampenv->labelfont(0);
            ampenv->labelsize(12);
            ampenv->labelcolor(FL_FOREGROUND_COLOR);
            ampenv->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            ampenv->when(FL_WHEN_RELEASE);
            o->init(pars->AmpEnvelope, npart, kititem, 2, 0);
            ampenv->end();
          } // EnvelopeUI* ampenv
          { LFOUI* o = amplfo = new LFOUI(10, 165, 230, 70, "Amplitude LFO     ");
            amplfo->box(FL_FLAT_BOX);
            amplfo->color(FL_DARK1);
            amplfo->selection_color(FL_BACKGROUND_COLOR);
            amplfo->labeltype(FL_NORMAL_LABEL);
            amplfo->labelfont(1);
            amplfo->labelsize(12);
            amplfo->labelcolor(FL_FOREGROUND_COLOR);
            amplfo->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            amplfo->when(FL_WHEN_RELEASE);
            o->init(pars->AmpLfo, npart, kititem, 2, 0);
            amplfo->end();
          } // LFOUI* amplfo
          { Fl_Check_Button* o = stereo = new Fl_Check_Button(15, 249, 17, 15, " Stereo");
            stereo->box(FL_THIN_UP_BOX);
            stereo->down_box(FL_DOWN_BOX);
            stereo->color((Fl_Color)237);
            stereo->labelsize(11);
            stereo->callback((Fl_Callback*)cb_stereo);
            o->value(pars->PStereo);
          } // Fl_Check_Button* stereo
          o->end();
        } // Fl_Group* o
        { Fl_Group* o = new Fl_Group(245, 25, 285, 250, "FILTER");
          o->box(FL_THIN_UP_BOX);
          o->labelfont(1);
          o->labelsize(12);
          o->align(Fl_Align(FL_ALIGN_TOP|FL_ALIGN_INSIDE));
          { EnvelopeUI* o = filterenv = new EnvelopeUI(250, 130, 275, 70, "PADSynth - Filter Envelope");
            filterenv->box(FL_FLAT_BOX);
            filterenv->color((Fl_Color)51);
            filterenv->selection_color(FL_BACKGROUND_COLOR);
            filterenv->labeltype(FL_NORMAL_LABEL);
            filterenv->labelfont(0);
            filterenv->labelsize(12);
            filterenv->labelcolor(FL_FOREGROUND_COLOR);
            filterenv->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            filterenv->when(FL_WHEN_RELEASE);
            o->init(pars->FilterEnvelope, npart, kititem, 2, 2);
            filterenv->end();
          } // EnvelopeUI* filterenv
          { LFOUI* o = filterlfo = new LFOUI(250, 200, 230, 70, "Filter LFO     ");
            filterlfo->box(FL_FLAT_BOX);
            filterlfo->color(FL_DARK1);
            filterlfo->selection_color(FL_BACKGROUND_COLOR);
            filterlfo->labeltype(FL_NORMAL_LABEL);
            filterlfo->labelfont(1);
            filterlfo->labelsize(12);
            filterlfo->labelcolor(FL_FOREGROUND_COLOR);
            filterlfo->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            filterlfo->when(FL_WHEN_RELEASE);
            o->init(pars->FilterLfo, npart, kititem, 2, 2);
            filterlfo->end();
          } // LFOUI* filterlfo
          { FilterUI* o = filterui = new FilterUI(250, 55, 275, 75, "PADsynth - Filter");
            filterui->box(FL_FLAT_BOX);
            filterui->color(FL_LIGHT1);
            filterui->selection_color(FL_BACKGROUND_COLOR);
            filterui->labeltype(FL_NORMAL_LABEL);
            filterui->labelfont(0);
            filterui->labelsize(12);
            filterui->labelcolor(FL_FOREGROUND_COLOR);
            filterui->align(Fl_Align(FL_ALIGN_WRAP|FL_ALIGN_INSIDE));
            filterui->when(FL_WHEN_RELEASE);
            o->init(pars->GlobalFilter,&pars->PFilterVelocityScale,&pars->PFilterVelocityScaleFunction, npart, kititem, 2);
            filterui->end();
          } // FilterUI* filterui
          o->end();
        } // Fl_Group* o
        o->end();
      } // Fl_Group* o
      o->end();
    } // Fl_Tabs* o
    { Fl_Button* o = applybutton = new Fl_Button(324, 410, 123, 24, "Apply Changes");
      applybutton->box(FL_THIN_UP_BOX);
      applybutton->labelfont(1);
      applybutton->labelsize(12);
      applybutton->callback((Fl_Callback*)cb_applybutton);
      o->color(FL_RED);
    } // Fl_Button* applybutton
    { Fl_Button* o = new Fl_Button(466, 410, 61, 24, "Close");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Close);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(205, 430, 25, 15, "C");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(11);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_C);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(235, 430, 25, 15, "P");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(11);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_P);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(15, 410, 123, 24, "Export");
      o->tooltip("Export samples as wav file");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)51);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Export);
      o->align(Fl_Align(FL_ALIGN_WRAP));
    } // Fl_Button* o
    o->copy_label(synth->getGuiMaster()->setPartWindowTitle("PadSynth").c_str());
    o->position(synth->getGuiMaster()->padNoteX, synth->getGuiMaster()->padNoteY);
    padnotewindow->end();
  } // Fl_Double_Window* padnotewindow
  return padnotewindow;
}

void PADnoteUI::send_data(int control, float value, int type, int par2 ) {
  //
      unsigned char partnum = npart;
      unsigned char parameter = 0xff;
      if (par2 < 0xff)
      {
          partnum = 0xf0; // main
          parameter = 0x80 | npart;
      }
  
      collect_data(synth, value, (Fl::event_button() | type), control, partnum, kititem, 2, 0xff, parameter, par2);
}

void PADnoteUI::returns_update(CommandBlock *getData) {
  //
      float value = getData->data.value;
      bool wasFromHere = getData->data.type & 0x20;
      unsigned char control = getData->data.control;
      unsigned char par2 = getData->data.par2;
      float result;
      string name;
  
      switch(control)
      {
          case 0:
              volume->value(value);
              break;
  
          case 1:
              vsns->value(value);
              break;
  
          case 2:
              pan->value(value);
              randompan->value(value == 0);
              break;
  
          case 16:
              bwdial->value(value);
              result = powf(value / 1000.0f, 1.1f);
              result = powf(10.0f, result * 4.0f) * 0.25f;
              bwcents->value(result);
              cbwidget->do_callback();
              break;
  
          case 17:
              bwscale->value(lrint(value));
              cbwidget->do_callback();
              break;
  
          case 19:
              spectrummode->value(lrint(value));
              if (pars->Pmode == 0)
              {
                  bwprofilegroup->activate();
                  bwdial->activate();
                  bwcents->activate();
                  hprofile->activate();
                  hprofile->color(54);
                  bwscale->activate();
              }
              else
              {
                  bwprofilegroup->deactivate();
                  bwdial->deactivate();
                  bwcents->deactivate();
                  hprofile->deactivate();
                  hprofile->color(48);
                  bwscale->deactivate();
              }
              cbwidget->do_callback();
              break;
  
          case 32:
              detune->value(lrint(value));
              detunevalueoutput->do_callback();
              break;
  
          case 33:
              fixedfreqetdial->value(lrint(value));
              break;
  
          case 34:
              hz440->value(value > 0.5f);
              if (value <= 0.5f)
                  fixedfreqetdial->deactivate();
              else
                  fixedfreqetdial->activate();
              break;
  
          case 35:
              octave->value(lrint(value));
              break;
  
          case 36:
              detunetype->value(lrint(value));
              break;
  
          case 37:
              coarsedet->value(lrint(value));
              break;
  
          case 38:
              bend->value(lrint(value));
              break;
  
          case 39:
              offset->value(lrint(value));
              break;
  
          case 48:
              hrpospar1->value(lrint(value));
              overtonepos->redraw();
              cbwidget->do_callback();
              break;
  
          case 49:
              hrpospar2->value(lrint(value));
              overtonepos->redraw();
              cbwidget->do_callback();
              break;
  
          case 50:
              hrpospar3->value(lrint(value));
              overtonepos->redraw();
              cbwidget->do_callback();
              break;
  
          case 51:
              hrpostype->value(lrint(value));
              overtonepos->redraw();
              cbwidget->do_callback();
              break;
  
          case 64:
              hpbasepar1->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 65:
              hpfreqmult->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 66:
              hpmpar1->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 67:
              hpmfreq->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 68:
              hpwidth->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 69:
              hpbasetype->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 70:
              hponehalf->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 71:
              hpamppar1->value(value);
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 72:
              hpamppar2->value(value);
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 73:
              hpamptype->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 74:
              hpampmode->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 75:
              hpautoscale->value(lrint(value));
              hprofile->redraw();
              cbwidget->do_callback();
              break;
  
          case 80:
              qbasenote->value(lrint(value));
              cbwidget->do_callback();
              break;
  
          case 81:
              qsmpoct->value(lrint(value));
              cbwidget->do_callback();
              break;
  
          case 82:
              qoct->value(lrint(value));
              cbwidget->do_callback();
              break;
  
          case 83:
              qsamplesize->value(lrint(value));
              cbwidget->do_callback();
              break;
  
          case 94:
              name = miscMsgPop(par2);
              if (name.find("FAILED") == 1)
              {
                  if (wasFromHere)
                      fl_alert("Some samples have failed.");
              }
              break;
  
          case 104:
              applybutton->color(FL_GRAY);
              applybutton->redraw();
              if (oscui)
              {
                  oscui->applybutton->color(FL_GRAY);
                  oscui->applybutton->redraw();
              }
              if (resui)
              {
                  resui->applybutton->color(FL_GRAY);
                  resui->applybutton->redraw();
              }
              break;
  
          case 112:
              stereo->value(value > 0.5f);
              break;
  
          case 120:
              dpop->value(value);
              break;
  
          case 121:
              pstr->value(value);
              break;
  
          case 122:
              pt->value(value);
              break;
  
          case 123:
              pstc->value(value);
              break;
  
          case 124:
              pvel->value(value);
              break;
  
      }
}

PADnoteUI::PADnoteUI(PADnoteParameters *parameters, int npart_, int kititem_) {
  //
      synth = parameters->getSynthEngine();
      pars = parameters;
      npart = npart_;
      kititem = kititem_;
      oscui = NULL;
      resui = new ResonanceUI(pars->resonance, npart, kititem, 2);
      make_window();
}

PADnoteUI::~PADnoteUI() {
  //
      delete(oscui);
      delete(resui);
  
      padnotewindow->hide();
      delete(padnotewindow);
}
