// generated by Fast Light User Interface Designer (fluid) version 1.0304

#include "ResonanceUI.h"
// ResonanceUI.cc
// Original ZynAddSubFX author Nasca Octavian Paul
// Copyright (C) 2002-2005 Nasca Octavian Paul
// Copyright 2009-2010, Alan Calvert
// Copyright 2015-2017, Will Godfrey

// This file is part of yoshimi, which is free software: you can redistribute
// it and/or modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.

// yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.   See the GNU General Public License (version 2 or
// later) for more details.

// You should have received a copy of the GNU General Public License along with
// yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
// Street, Fifth Floor, Boston, MA  02110-1301, USA.

// This file is a derivative of the ZynAddSubFX original

// Modified October 2017

#include <cmath>
#include <iostream>
#include "MasterUI.h"

ResonanceGraph::ResonanceGraph(int x,int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label) {
  //
      respar=NULL;
      cbwidget=NULL;
      applybutton=NULL;
}

void ResonanceGraph::send_data(int control, float value, int type) {
  //
      collect_data(synth, value, (Fl::event_button() | type), control, npart, kititem, engine, 9);
}

void ResonanceGraph::init(Resonance *respar_, Fl_Value_Output *khzvalue_, Fl_Value_Output *dbvalue_, int npart_, int kititem_, int engine_) {
  //
      respar=respar_;
      synth = respar->getSynthEngine();
      npart = npart_;
      kititem = kititem_;
      engine = engine_;
      khzvalue=khzvalue_;
      dbvalue=dbvalue_;
      oldx=-1;
      khzval=-1;
}

void ResonanceGraph::draw_freq_line(float freq,int type) {
  //
      float freqx=respar->getfreqpos(freq);
      switch(type){
          case 0:
              fl_line_style(FL_SOLID);
              break;
          case 1:
              fl_line_style(FL_DOT);
              break;
          case 2:
              fl_line_style(FL_DASH);
              break;
      }
  
      if ((freqx>0.0)&&(freqx<1.0))
          fl_line(x()+(int) (freqx*w()),y(),
      x()+(int) (freqx*w()),y()+h());
}

void ResonanceGraph::draw() {
  //
      int ox=x();
      int oy=y();
      int lx=w();
      int ly=h();
      int i;
      int ix;
      int iy;
      int oiy;
      float freqx;
  
      fl_color(FL_BLACK);
      fl_rectf(ox,oy,lx,ly);
  
  
      //draw the lines
      fl_color(FL_GRAY);
  
      fl_line_style(FL_SOLID);
      fl_line(ox+2,oy+ly/2,ox+lx-2,oy+ly/2);
  
      freqx=respar->getfreqpos(1000.0);
      if ((freqx>0.0)&&(freqx<1.0))
          fl_line(ox+(int) (freqx*lx),oy, ox+(int) (freqx*lx),oy+ly);
  
      for (i=1;i<10;i++)
      {
          if(i==1)
          {
              draw_freq_line(i*100.0,0);
              draw_freq_line(i*1000.0,0);
          }
          else if (i==5)
          {
              draw_freq_line(i*100.0,2);
              draw_freq_line(i*1000.0,2);
          }
          else
          {
              draw_freq_line(i*100.0,1);
              draw_freq_line(i*1000.0,1);
          }
      }
  
      draw_freq_line(10000.0,0);
      draw_freq_line(20000.0,1);
  
      fl_line_style(FL_DOT);
      int GY=10;
      if (ly<GY*3)
          GY=-1;
      for (i=1;i<GY;i++)
      {
          int tmp=(int)(ly/(float)GY*i);
          fl_line(ox+2,oy+tmp,ox+lx-2,oy+tmp);
      }
  
      //draw the data
      fl_color(FL_RED);
      fl_line_style(FL_SOLID);
      oiy=(int)(respar->Prespoints[0]/128.0*ly);
      for (i=1;i<MAX_RESONANCE_POINTS;i++)
      {
          ix=(int)(i*1.0/MAX_RESONANCE_POINTS*lx);
          iy=(int)(respar->Prespoints[i]/128.0*ly);
          fl_line(ox+ix-1,oy+ly-oiy,ox+ix,oy+ly-iy);
          oiy=iy;
      }
}

int ResonanceGraph::handle(int event) {
  //
      int x_ = Fl::event_x() - x();
      int y_ = Fl::event_y() - y();
  
      if ( (x_ >= 0) && (x_ < w()) && (y_ >= 0) && (y_ < h()))
      {
          khzvalue->value(respar->getfreqx(x_ * 1.0 / w()) / 1000.0);
          dbvalue->value((1.0 - y_ * 2.0 / h()) * respar->PmaxdB);
      }
  
      if ((event == FL_PUSH) || (event == FL_DRAG))
      {
          unsigned char typeSent;
          if (event == FL_DRAG)
              typeSent = 0xc0;
          else
              typeSent = 0xd0;
          int leftbutton = 1;
          if (Fl::event_button() == FL_RIGHT_MOUSE)
              leftbutton = 0;
          if (x_ < 0)
              x_ = 0;
          if (y_ < 0)
              y_ = 0;
          if (x_ >= w())
              x_ = w();
          if (y_ >= h() -1)
              y_ = h() -1;
  
          if ((oldx < 0) || (oldx == x_))
          {
              int sn = (int)(x_ * 1.0 / w() * MAX_RESONANCE_POINTS);
              int sp=127 - (int)(y_ * 1.0 / h() * 127);
              if (leftbutton != 0)
              {
                  send_data(sn, sp, typeSent);
              }
              else
              {
                  send_data(sn, 64, typeSent);
              }
          }
          else
          {
              int x1 = oldx;
              int x2 = x_;
              int y1 = oldy;
              int y2 = y_;
              if (oldx > x_)
              {
                  x1 = x_;
                  y1 = y_;
                  x2 = oldx;
                  y2 = oldy;
              }
              for (int i = 0; i < x2 - x1; i++)
              {
                  int sn = (int)((i + x1) * 1.0 / w() * MAX_RESONANCE_POINTS);
                  float yy = (y2 - y1) * 1.0 / (x2 - x1) * i;
                  int sp = 127 - (int)((y1 + yy) / h() * 127);
                  if (leftbutton!=0)
                  {
                      send_data(sn, sp, typeSent);
                  }
                  else
                  {
                  send_data(sn, 64, typeSent);
                  }
              }
          }
          oldx = x_;
          oldy = y_;
          redraw();
      }
  
      if (event == FL_RELEASE)
      {
          oldx = -1;
          if (cbwidget != NULL)
          {
              cbwidget->do_callback();
              if (applybutton != NULL)
              {
                  applybutton->color(FL_RED);
                  applybutton->redraw();
              }
          }
      }
  
      return(1);
}

void ResonanceGraph::setcbwidget(Fl_Widget *cbwidget,Fl_Widget *applybutton) {
  //
      this->cbwidget=cbwidget;
      this->applybutton=applybutton;
}

void ResonanceUI::cb_Close_i(Fl_Button*, void*) {
  //
        resonancewindow->hide();
        if ((Fl::event_button() == 3))
        {
            if (ADvsPAD)
                synth->getGuiMaster()->partui->adnoteui->ADnoteGlobalParameters->show();
            else
                synth->getGuiMaster()->partui->padnoteui->padnotewindow->show();
        };
}
void ResonanceUI::cb_Close(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_Close_i(o,v);
}

void ResonanceUI::cb_Clear_i(Fl_Button* o, void*) {
  //
        send_data(96, o->value(), 0xd0);
}
void ResonanceUI::cb_Clear(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_Clear_i(o,v);
}

void ResonanceUI::cb_Smooth_i(Fl_Button* o, void*) {
  //
        send_data(97, o->value(), 0xd0);
}
void ResonanceUI::cb_Smooth(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_Smooth_i(o,v);
}

void ResonanceUI::cb_enabled_i(Fl_Check_Button* o, void*) {
  //
        send_data(8, o->value(), 0xd0);
}
void ResonanceUI::cb_enabled(Fl_Check_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_enabled_i(o,v);
}

void ResonanceUI::cb_maxdbvo_i(Fl_Value_Output* o, void*) {
  //
        o->value(respar->PmaxdB);
}
void ResonanceUI::cb_maxdbvo(Fl_Value_Output* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_maxdbvo_i(o,v);
}

void ResonanceUI::cb_maxdb_i(mwheel_slider_rev* o, void*) {
  //
        if (Fl::event_button() == 3)
            o->value(20);

        send_data(0, o->value(), 0xd8);
}
void ResonanceUI::cb_maxdb(mwheel_slider_rev* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_maxdb_i(o,v);
}

void ResonanceUI::cb_centerfreqvo_i(Fl_Value_Output* o, void*) {
  o->value(respar->getcenterfreq()/1000.0);
}
void ResonanceUI::cb_centerfreqvo(Fl_Value_Output* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_centerfreqvo_i(o,v);
}

void ResonanceUI::cb_centerfreq_i(mwheel_slider_rev* o, void*) {
  //
        if (Fl::event_button() == 3)
            o->value(64);

        send_data(1, o->value(), 0xd0);
}
void ResonanceUI::cb_centerfreq(mwheel_slider_rev* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_centerfreq_i(o,v);
}

void ResonanceUI::cb_octavesfreqvo_i(Fl_Value_Output* o, void*) {
  o->value(respar->getoctavesfreq());
}
void ResonanceUI::cb_octavesfreqvo(Fl_Value_Output* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_octavesfreqvo_i(o,v);
}

void ResonanceUI::cb_octavesfreq_i(mwheel_slider_rev* o, void*) {
  //
        if (Fl::event_button() == 3)
            o->value(64);

        send_data(2, o->value(), 0xd0);
}
void ResonanceUI::cb_octavesfreq(mwheel_slider_rev* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_octavesfreq_i(o,v);
}

void ResonanceUI::cb_RND1_i(Fl_Button*, void*) {
  //
        send_data(10, 0, 0xd0);
}
void ResonanceUI::cb_RND1(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_RND1_i(o,v);
}

void ResonanceUI::cb_RND2_i(Fl_Button*, void*) {
  //
        send_data(10, 1, 0xd0);
}
void ResonanceUI::cb_RND2(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_RND2_i(o,v);
}

void ResonanceUI::cb_RND3_i(Fl_Button*, void*) {
  //
        send_data(10, 2, 0xd0);
}
void ResonanceUI::cb_RND3(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_RND3_i(o,v);
}

void ResonanceUI::cb_p1st_i(Fl_Check_Button* o, void*) {
  //
        send_data(21, o->value(), 0xd0);
}
void ResonanceUI::cb_p1st(Fl_Check_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_p1st_i(o,v);
}

void ResonanceUI::cb_S_i(Fl_Button*, void*) {
  //
        send_data(20, 0, 0xd0);
}
void ResonanceUI::cb_S(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_S_i(o,v);
}

void ResonanceUI::cb_L_i(Fl_Button*, void*) {
  //
        send_data(20, 1, 0xd0);
}
void ResonanceUI::cb_L(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_L_i(o,v);
}

void ResonanceUI::cb_C_i(Fl_Button*, void*) {
  synth->getGuiMaster()->getPresetsUi()->copy(respar);
}
void ResonanceUI::cb_C(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_C_i(o,v);
}

void ResonanceUI::cb_P_i(Fl_Button*, void*) {
  synth->getGuiMaster()->getPresetsUi()->paste(respar,this);
}
void ResonanceUI::cb_P(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_P_i(o,v);
}

void ResonanceUI::cb_applybutton_i(Fl_Button*, void*) {
  //
            send_data(104, 0, 0xd0);
}
void ResonanceUI::cb_applybutton(Fl_Button* o, void* v) {
  ((ResonanceUI*)(o->parent()->user_data()))->cb_applybutton_i(o,v);
}

Fl_Double_Window* ResonanceUI::make_window() {
  { resonancewindow = new Fl_Double_Window(780, 305, "Resonance");
    resonancewindow->user_data((void*)(this));
    { Fl_Value_Output* o = khzvalue = new Fl_Value_Output(435, 264, 40, 18, "kHz");
      khzvalue->labelsize(12);
      khzvalue->minimum(0.001);
      khzvalue->maximum(48);
      khzvalue->textfont(1);
      khzvalue->textsize(12);
      khzvalue->align(Fl_Align(FL_ALIGN_RIGHT));
      o->step(0.01, 1);
      //this widget must be before the calling widgets
    } // Fl_Value_Output* khzvalue
    { Fl_Value_Output* o = dbvalue = new Fl_Value_Output(435, 282, 40, 18, "dB");
      dbvalue->labelsize(12);
      dbvalue->minimum(-150);
      dbvalue->maximum(150);
      dbvalue->textfont(1);
      dbvalue->textsize(12);
      dbvalue->align(Fl_Align(FL_ALIGN_RIGHT));
      o->step(0.1, 1);
      //this widget must be before the calling widgets
    } // Fl_Value_Output* dbvalue
    { Fl_Group* o = new Fl_Group(6, 5, 768, 256);
      o->box(FL_BORDER_BOX);
      rg=new ResonanceGraph(o->x(),o->y(),o->w(),o->h(),"");
      rg->init(respar, khzvalue, dbvalue, npart, kititem, engine);
      rg->show();
      o->end();
    } // Fl_Group* o
    { Fl_Button* o = new Fl_Button(700, 285, 74, 16, "Close");
      o->box(FL_THIN_UP_BOX);
      o->callback((Fl_Callback*)cb_Close);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(507, 264, 66, 15, "Clear");
      o->tooltip("Clear the resonance function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Clear);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(507, 282, 66, 18, "Smooth");
      o->tooltip("Smooth the resonance function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Smooth);
    } // Fl_Button* o
    { Fl_Check_Button* o = enabled = new Fl_Check_Button(6, 265, 78, 15, "Enable");
      enabled->down_box(FL_DOWN_BOX);
      enabled->labelfont(1);
      enabled->labelsize(12);
      enabled->callback((Fl_Callback*)cb_enabled);
      o->value(respar->Penabled);
    } // Fl_Check_Button* enabled
    { Fl_Value_Output* o = maxdbvo = new Fl_Value_Output(55, 282, 22, 18, "Max dB");
      maxdbvo->tooltip("The Maximum amplitude (dB)");
      maxdbvo->box(FL_THIN_DOWN_BOX);
      maxdbvo->labelsize(12);
      maxdbvo->minimum(1);
      maxdbvo->maximum(127);
      maxdbvo->step(1);
      maxdbvo->value(30);
      maxdbvo->textfont(1);
      maxdbvo->textsize(12);
      maxdbvo->callback((Fl_Callback*)cb_maxdbvo);
      o->value(respar->PmaxdB);
    } // Fl_Value_Output* maxdbvo
    { mwheel_slider_rev* o = maxdb = new mwheel_slider_rev(77, 282, 98, 18);
      maxdb->type(5);
      maxdb->box(FL_THIN_DOWN_BOX);
      maxdb->color(FL_BACKGROUND_COLOR);
      maxdb->selection_color(FL_BACKGROUND_COLOR);
      maxdb->labeltype(FL_NORMAL_LABEL);
      maxdb->labelfont(0);
      maxdb->labelsize(11);
      maxdb->labelcolor(FL_FOREGROUND_COLOR);
      maxdb->minimum(1);
      maxdb->maximum(90);
      maxdb->step(1);
      maxdb->value(20);
      maxdb->callback((Fl_Callback*)cb_maxdb);
      maxdb->align(Fl_Align(FL_ALIGN_RIGHT));
      maxdb->when(FL_WHEN_CHANGED);
      o->value(respar->PmaxdB);
    } // mwheel_slider_rev* maxdb
    { Fl_Value_Output* o = centerfreqvo = new Fl_Value_Output(210, 264, 33, 18, "C.f.");
      centerfreqvo->tooltip("Center Frequency (kHz)");
      centerfreqvo->labelsize(12);
      centerfreqvo->minimum(1);
      centerfreqvo->maximum(10);
      centerfreqvo->value(1);
      centerfreqvo->textfont(1);
      centerfreqvo->textsize(12);
      centerfreqvo->callback((Fl_Callback*)cb_centerfreqvo);
      centerfreqvo->when(3);
      o->step(0.01, 1);
      o->value(respar->getcenterfreq()/1000.0);
    } // Fl_Value_Output* centerfreqvo
    { mwheel_slider_rev* o = centerfreq = new mwheel_slider_rev(243, 264, 110, 18);
      centerfreq->type(5);
      centerfreq->box(FL_THIN_DOWN_BOX);
      centerfreq->color(FL_BACKGROUND_COLOR);
      centerfreq->selection_color(FL_BACKGROUND_COLOR);
      centerfreq->labeltype(FL_NORMAL_LABEL);
      centerfreq->labelfont(0);
      centerfreq->labelsize(11);
      centerfreq->labelcolor(FL_FOREGROUND_COLOR);
      centerfreq->maximum(127);
      centerfreq->step(1);
      centerfreq->callback((Fl_Callback*)cb_centerfreq);
      centerfreq->align(Fl_Align(FL_ALIGN_RIGHT));
      centerfreq->when(FL_WHEN_CHANGED);
      o->value(respar->Pcenterfreq);
    } // mwheel_slider_rev* centerfreq
    { Fl_Value_Output* o = octavesfreqvo = new Fl_Value_Output(210, 282, 33, 18, "Oct.");
      octavesfreqvo->tooltip("No. of octaves");
      octavesfreqvo->labelsize(12);
      octavesfreqvo->minimum(1);
      octavesfreqvo->maximum(127);
      octavesfreqvo->step(1);
      octavesfreqvo->value(5);
      octavesfreqvo->textfont(1);
      octavesfreqvo->textsize(12);
      octavesfreqvo->callback((Fl_Callback*)cb_octavesfreqvo);
      octavesfreqvo->when(3);
      o->value(respar->getoctavesfreq());
    } // Fl_Value_Output* octavesfreqvo
    { mwheel_slider_rev* o = octavesfreq = new mwheel_slider_rev(243, 282, 110, 18);
      octavesfreq->type(5);
      octavesfreq->box(FL_THIN_DOWN_BOX);
      octavesfreq->color(FL_BACKGROUND_COLOR);
      octavesfreq->selection_color(FL_BACKGROUND_COLOR);
      octavesfreq->labeltype(FL_NORMAL_LABEL);
      octavesfreq->labelfont(0);
      octavesfreq->labelsize(11);
      octavesfreq->labelcolor(FL_FOREGROUND_COLOR);
      octavesfreq->maximum(127);
      octavesfreq->step(1);
      octavesfreq->callback((Fl_Callback*)cb_octavesfreq);
      octavesfreq->align(Fl_Align(FL_ALIGN_RIGHT));
      octavesfreq->when(FL_WHEN_CHANGED);
      o->value(respar->Poctavesfreq);
    } // mwheel_slider_rev* octavesfreq
    { Fl_Button* o = new Fl_Button(583, 264, 42, 12, "RND1");
      o->tooltip("Randomize the resonance function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(10);
      o->callback((Fl_Callback*)cb_RND1);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(583, 276, 42, 12, "RND2");
      o->tooltip("Randomize the resonance function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(10);
      o->callback((Fl_Callback*)cb_RND2);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(583, 288, 42, 12, "RND3");
      o->tooltip("Randomize the resonance function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(10);
      o->callback((Fl_Callback*)cb_RND3);
    } // Fl_Button* o
    { Fl_Check_Button* o = p1st = new Fl_Check_Button(106, 265, 45, 15, "Prot.1st");
      p1st->tooltip("Protect the fundamental frequency (do not damp the first harmonic)");
      p1st->down_box(FL_DOWN_BOX);
      p1st->labelsize(12);
      p1st->callback((Fl_Callback*)cb_p1st);
      o->value(respar->Pprotectthefundamental);
    } // Fl_Check_Button* p1st
    { Fl_Text_Display* o = new Fl_Text_Display(364, 280, 62, 0, "InterpPk");
      o->box(FL_NO_BOX);
      o->labelsize(13);
      o->textsize(13);
      o->align(Fl_Align(33));
    } // Fl_Text_Display* o
    { Fl_Box* o = new Fl_Box(359, 264, 72, 36);
      o->tooltip("Interpolate the peaks (left button smooth, right button linear)");
      o->box(FL_THIN_DOWN_FRAME);
    } // Fl_Box* o
    { Fl_Button* o = new Fl_Button(365, 282, 14, 14, "S");
      o->tooltip("Smooth interpolation");
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_S);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(411, 282, 14, 14, "L");
      o->tooltip("Linear interpolation");
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_L);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(635, 275, 25, 15, "C");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(11);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_C);
    } // Fl_Button* o
    { Fl_Button* o = new Fl_Button(665, 275, 25, 15, "P");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(11);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_P);
    } // Fl_Button* o
    { applybutton = new Fl_Button(700, 265, 74, 17, "Apply");
      applybutton->box(FL_THIN_UP_BOX);
      applybutton->down_box(FL_THIN_DOWN_BOX);
      applybutton->labelfont(1);
      applybutton->labelsize(11);
      applybutton->callback((Fl_Callback*)cb_applybutton);
    } // Fl_Button* applybutton
    resonancewindow->end();
  } // Fl_Double_Window* resonancewindow
  return resonancewindow;
}

void ResonanceUI::send_data(int control, float value, int type) {
  //
      int insert;
      if (control == 104)
          insert = 0xff;
      else
          insert = 8;
  
      collect_data(synth, value, (type | Fl::event_button()), control, npart, kititem, engine, insert);
}

void ResonanceUI::returns_update(CommandBlock *getData) {
  //
      float value = getData->data.value;
      unsigned char control = getData->data.control;
      unsigned char eng = getData->data.engine;
      unsigned char insert = getData->data.insert;
  
      if (eng != engine)
          return;
  
      bool set = true;
      if (insert == 9)
      {
          rg->draw();
          rg->redraw();
      }
      else
      {
          switch(control)
          {
              case 0:
                  maxdb->value(value);
                  maxdbvo->value(value);
                  break;
  
              case 1:
                  centerfreq->value(value);
                  value = 10000.0 * powf(10.0f, - (1.0f - value / 127.0f) * 2.0f);
                  centerfreqvo->value(value/1000.0);
                  rg->redraw();
                  break;
  
              case 2:
                  octavesfreq->value(value);
                  octavesfreqvo->value(0.25 + 10.0 * value / 127.0);
                  rg->redraw();
                  break;
  
              case 8:
                  enabled->value(value != 0);
                  break;
  
              case 10:
                  resonancewindow->redraw();
                  break;
  
              case 20:
                  resonancewindow->redraw();
                  break;
  
              case 21:
                  p1st->value(value != 0);
                  break;
  
              case 96:
                  resonancewindow->redraw();
                  break;
  
              case 97:
                  resonancewindow->redraw();
                  break;
  
              default:
                  set = false;
          }
      }
  
      if (set)
      {
          if (eng == 2)
              redrawPADnoteApply();
      }
}

ResonanceUI::ResonanceUI(Resonance *respar_, int npart_, int kititem_, int engine_) {
  //
      respar = respar_;
      synth = respar->getSynthEngine();
      npart = npart_;
      kititem = kititem_;
      engine = engine_;
          cbwidget = NULL;
          cbapplywidget = NULL;
          make_window();
          applybutton->hide();
}

ResonanceUI::~ResonanceUI() {
  resonancewindow->hide();
}

void ResonanceUI::redrawPADnoteApply() {
  if (cbwidget!=NULL) {
      cbwidget->do_callback();
      applybutton->color(FL_RED);
      applybutton->redraw();
  };
}

void ResonanceUI::setcbwidget(Fl_Widget *cbwidget,Fl_Widget *cbapplywidget) {
  //
      this->cbwidget = cbwidget;
      this->cbapplywidget = cbapplywidget;
      rg->setcbwidget(cbwidget, applybutton);
      applybutton->show();
}

void ResonanceUI::refresh() {
  //
      redrawPADnoteApply();
      enabled->value(respar->Penabled);
      maxdb->value(respar->PmaxdB);
      maxdbvo->value(respar->PmaxdB);
      centerfreqvo->value(respar->getcenterfreq()/1000.0);
      octavesfreqvo->value(respar->getoctavesfreq());
      centerfreq->value(respar->Pcenterfreq);
      octavesfreq->value(respar->Poctavesfreq);
      p1st->value(respar->Pprotectthefundamental);
      rg->redraw();
}

void ResonanceUI::Show(bool _ADvsPAD) {
  //
      ADvsPAD = _ADvsPAD;
      string tname;
      if (engine == 0)
          tname = "AddSynth Resonance";
      else
          tname = "PadSynth Resonance ";
      resonancewindow->copy_label(synth->getGuiMaster()->setPartWindowTitle(tname).c_str());
      resonancewindow->show();
}
